import { useEffect, useRef, useState } from "react";
import Button from "./Button";
import GetUsersInProject, { ProjectMember } from "./GetUsersInProject";
import { Link } from "react-router-dom";
import GetProjectTimes, { projectTimes } from "./GetProjectTimes";
import DeleteProject from "./DeleteProject";
import InputField from "./InputField";
import ProjectNameInput from "./Inputs/ProjectNameInput";
import { alphanumeric } from "../Data/regex";
import { projNameHighLimit, projNameLowLimit } from "../Data/constants";

function ProjectInfoModal(props: {
  projectname: string;
  onClose: () => void;
  onClick: (username: string, userRole: string) => void;
}): JSX.Element {
  const [showInput, setShowInput] = useState(false);
  const [users, setUsers] = useState<ProjectMember[]>([]);
  const [times, setTimes] = useState<projectTimes>();
  const [search, setSearch] = useState("");
  const [newProjName, setNewProjName] = useState("");
  const totalTime = useRef(0);
  GetUsersInProject({ projectName: props.projectname, setUsersProp: setUsers });

  GetProjectTimes({ setTimesProp: setTimes, projectName: props.projectname });

  const handleChangeNameView = (): void => {
    if (showInput) {
      setNewProjName("");
      setShowInput(false);
    } else {
      setShowInput(true);
    }
  };

  const handleClickChangeName = (): void => {
    if (
      newProjName.length > projNameHighLimit ||
      newProjName.length < projNameLowLimit ||
      !alphanumeric.test(newProjName)
    ) {
      alert(
        "Please provide valid project name: \n-Between 10-99 characters \n-No special characters (.-!?/*)",
      );
      return;
    }

    if (
      confirm(
        `Are you sure you want to change name of ${props.projectname} to ${newProjName}?`,
      )
    ) {
      //TODO: change and insert change name functionality
      alert("Not implemented yet");
      setNewProjName("");
    } else {
      alert("Name was not changed!");
    }
  };

  useEffect(() => {
    if (times?.totalTime !== undefined) {
      totalTime.current = times.totalTime;
    }
  }, [times]);

  return (
    <div
      className="fixed inset-0 bg-black bg-opacity-30 backdrop-blur-sm 
      flex justify-center items-center"
    >
      <div className="border-4 border-black bg-white p-2 rounded-2xl text-center h-[64vh] w-[40] overflow-auto">
        <div className="pl-10 pr-10">
          <h1 className="font-bold text-[32px]">{props.projectname}</h1>
          <p
            className="mb-4 hover:font-bold hover:cursor-pointer hover:underline"
            onClick={handleChangeNameView}
          >
            (Change project name)
          </p>
          {showInput && (
            <>
              <h2 className="text-[20px] font-bold pb-2">Change name:</h2>
              <div className="border-2 rounded-2xl border-black px-6 pt-6 pb-1 mb-7">
                <ProjectNameInput
                  name={newProjName}
                  onChange={function (e): void {
                    setNewProjName(e.target.value);
                  }}
                />
                <div className="px-6 grid grid-cols-2 gap-10">
                  <Button
                    text={"Change"}
                    onClick={function (): void {
                      handleClickChangeName();
                    }}
                    type={"submit"}
                  />
                  <Button
                    text={"Close"}
                    onClick={function (): void {
                      handleChangeNameView();
                    }}
                    type={"submit"}
                  />
                </div>
              </div>
            </>
          )}
          <h2 className="text-[20px] font-bold pb-2">Statistics:</h2>
          <div className="border-2 border-black rounded-2xl px-2 py-1 text-left divide-y-2 flex flex-col overflow-auto">
            <p>Number of members: {users.length}</p>
            <p>
              Total time reported:{" "}
              {Math.floor(totalTime.current / 60 / 24) + " d "}
              {Math.floor((totalTime.current / 60) % 24) + " h "}
              {(totalTime.current % 60) + " m "}
            </p>
          </div>
          <h3 className="pt-7 text-[20px] font-bold">Project members:</h3>
          <div className="">
            <InputField
              placeholder={"Search member"}
              type={"Text"}
              value={search}
              onChange={(e) => {
                setSearch(e.target.value);
              }}
            />
            <ul className="border-2 border-black mt-2 p-2 rounded-2xl text-left overflow-auto h-[24vh] font-medium space-y-2">
              {users
                .filter((user) => {
                  return search.toLowerCase() === ""
                    ? user.Username
                    : user.Username.toLowerCase().includes(
                        search.toLowerCase(),
                      );
                })
                .map((user) => (
                  <li
                    className="items-start px-2 py-1 border-2 border-black rounded-2xl bg-orange-200 transition-all hover:bg-orange-600 hover:text-white hover:cursor-pointer"
                    key={user.Username}
                    onClick={() => {
                      props.onClick(user.Username, user.UserRole);
                    }}
                  >
                    <span>
                      Name: {user.Username}
                      <div></div>
                      Role: {user.UserRole}
                    </span>
                  </li>
                ))}
            </ul>
          </div>
          <div className="space-x-5 my-2">
            <Button
              text={"Delete"}
              onClick={function (): void {
                if (
                  window.confirm(
                    "Are you sure you want to delete this project?",
                  )
                ) {
                  DeleteProject({
                    projectToDelete: props.projectname,
                  });
                }
              }}
              type="button"
            />
            <Link
              to={{
                pathname: "/adminProjectAddMember",
                search: props.projectname,
              }}
            >
              <Button
                text={"Add Member"}
                onClick={function (): void {
                  return;
                }}
                type="button"
              />
            </Link>
            <Button
              text={"Close"}
              onClick={function (): void {
                props.onClose();
              }}
              type="button"
            />
          </div>
        </div>
      </div>
    </div>
  );
}

export default ProjectInfoModal;
