import Button from "./Button";
import DeleteUser from "./DeleteUser";
import UserProjectListAdmin from "./UserProjectListAdmin";
import { useState } from "react";
import ChangeUsername from "./ChangeUsername";
import { StrNameChange } from "../Types/goTypes";
import UsernameInput from "./Inputs/UsernameInput";
import PasswordInput from "./Inputs/PasswordInput";
import { alphanumeric, lowercase } from "../Data/regex";
import {
  passwordLength,
  usernameLowLimit,
  usernameUpLimit,
} from "../Data/constants";

function UserInfoModal(props: {
  isVisible: boolean;
  username: string;
  onClose: () => void;
}): JSX.Element {
  const [showNameInput, setShowNameInput] = useState(false);
  const [showPwordInput, setShowPwordInput] = useState(false);
  const [newUsername, setNewUsername] = useState("");
  const [newPassword, setNewPassword] = useState("");
  if (!props.isVisible) {
    return <></>;
  }

  /*
   * Switches name input between visible/invisible
   * and makes password input invisible
   */
  const handleShowNameInput = (): void => {
    if (showPwordInput) setShowPwordInput(false);
    if (showNameInput) {
      setShowNameInput(false);
      setNewUsername("");
    } else {
      setShowNameInput(true);
      setNewPassword("");
    }
  };

  /*
   * Switches password input between visible/invisible
   * and makes username input invisible
   */
  const handleShowPwordInput = (): void => {
    if (showNameInput) setShowNameInput(false);
    if (showPwordInput) {
      setShowPwordInput(false);
      setNewPassword("");
    } else {
      setShowPwordInput(true);
      setNewUsername("");
    }
  };

  // Handles name change and checks if new name meets requirements
  const handleClickChangeName = (): void => {
    if (
      !alphanumeric.test(newUsername) ||
      newUsername.length > usernameUpLimit ||
      newUsername.length < usernameLowLimit
    ) {
      alert(
        "Please provide valid username: \n-Between 5-10 characters \n-No special characters (.-!?/*)",
      );
      return;
    }
    if (
      confirm(
        `Do you really want to change username of ${props.username} to ${newUsername}?`,
      )
    ) {
      const nameChange: StrNameChange = {
        prevName: props.username,
        newName: newUsername.replace(/ /g, ""),
      };
      ChangeUsername({ nameChange: nameChange });
    } else {
      alert("Name was not changed!");
    }
  };

  // Handles password change and checks if new password meets requirements
  const handleClickChangePassword = (): void => {
    if (newPassword.length !== passwordLength || !lowercase.test(newPassword)) {
      alert(
        "Please provide valid password: \n-Exactly 6 characters \n-No uppercase letters \n-No numbers \n-No special characters (.-!?/*)",
      );
      return;
    }
    if (
      confirm(`Are you sure you want to change password of ${props.username}?`)
    ) {
      //TODO: insert change password functionality
      alert("Not implemented yet");
      setNewPassword("");
    } else {
      alert("Password was not changed!");
    }
  };

  return (
    <div
      className="fixed inset-0 bg-black bg-opacity-30 backdrop-blur-sm 
      flex justify-center items-center"
    >
      <div className="border-4 border-black bg-white rounded-2xl text-center flex flex-col">
        <div className="mx-10">
          <p className="font-bold text-[30px]">{props.username}</p>
          <p className="mt-2 font-bold text-[20px]">Change:</p>
          <p className="mt-2 space-x-3 mb-[10px]">
            <span
              className={
                showNameInput
                  ? "items-start font-semibold py-1 px-2 border-2 border-transparent rounded-full bg-orange-500 transition-all hover:bg-orange-600  text-white hover:cursor-pointer ring-2 ring-black"
                  : "items-start font-medium py-1 px-2 border-2 border-gray-500 text-white rounded-full bg-orange-300 hover:bg-orange-400 transition-all hover:text-gray-100 hover:border-gray-600 hover:cursor-pointer"
              }
              onClick={handleShowNameInput}
            >
              Username
            </span>{" "}
            <span
              className={
                showPwordInput
                  ? "items-start font-semibold py-1 px-2 border-2 border-transparent rounded-full bg-orange-500 transition-all hover:bg-orange-600  text-white hover:cursor-pointer ring-2 ring-black"
                  : "items-start font-medium py-1 px-2 border-2 border-gray-500 text-white rounded-full bg-orange-300 hover:bg-orange-400 transition-all hover:text-gray-100 hover:border-gray-600 hover:cursor-pointer"
              }
              onClick={handleShowPwordInput}
            >
              Password
            </span>
          </p>
          {showNameInput && (
            <div className="mt-7">
              <UsernameInput
                username={newUsername}
                onChange={(e) => {
                  setNewUsername(e.target.value);
                }}
              />
              <Button
                text={"Change"}
                onClick={function (): void {
                  handleClickChangeName();
                }}
                type={"submit"}
              />
            </div>
          )}
          {showPwordInput && (
            <div className="mt-7">
              <PasswordInput
                password={newPassword}
                onChange={(e) => {
                  setNewPassword(e.target.value);
                }}
              />
              <Button
                text={"Change"}
                onClick={function (): void {
                  handleClickChangePassword();
                }}
                type={"submit"}
              />
            </div>
          )}
          <h2 className="font-bold text-[20px]">Member of these projects:</h2>
          <UserProjectListAdmin username={props.username} />
          <div className="items-center space-x-6">
            <Button
              text={"Delete"}
              onClick={function (): void {
                if (
                  window.confirm("Are you sure you want to delete this user?")
                ) {
                  DeleteUser({
                    usernameToDelete: props.username,
                  });
                }
              }}
              type="button"
            />
            <Button
              text={"Close"}
              onClick={function (): void {
                setNewUsername("");
                setNewPassword("");
                setShowNameInput(false);
                setShowPwordInput(false);
                props.onClose();
              }}
              type="button"
            />
          </div>
        </div>
      </div>
    </div>
  );
}

export default UserInfoModal;
